@@ -17,9 +17,9 @@
 DOCUMENTATION = '''
 ---
 module: ec2
-short_description: create, terminate, start or stop an instance in ec2, return instanceid
+short_description: create, terminate, start or stop an instance in ec2
 description:
-    - Creates or terminates ec2 instances. When created optionally waits for it to be 'running'. This module has a dependency on python-boto >= 2.5
+    - Creates or terminates ec2 instances.
 version_added: "0.9"
 options:
   key_name:
@@ -28,12 +28,6 @@ options:
     required: false
     default: null
     aliases: ['keypair']
-  id:
-    description:
-      - identifier for this instance or set of instances, so that the module will be idempotent with respect to EC2 instances. This identifier is valid for at least 24 hours after the termination of the instance, and should not be reused for another call later on. For details, see the description of client token at U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html).
-    required: false
-    default: null
-    aliases: []
   group:
     description:
       - security group (or list of groups) to use with the instance
@@ -67,6 +61,13 @@ options:
     required: true
     default: null
     aliases: []
+  tenancy:
+    version_added: "1.9"
+    description:
+      - An instance with a tenancy of "dedicated" runs on single-tenant hardware and can only be launched into a VPC. Valid values are "default" or "dedicated". Note that to use dedicated tenancy you MUST specify a vpc_subnet_id as well. Dedicated tenancy is not available for EC2 "micro" instances.
+    required: false
+    default: default
+    aliases: []
   spot_price:
     version_added: "1.5"
     description:
@@ -76,7 +77,7 @@ options:
     aliases: []
   image:
     description:
-       - I(emi) (or I(ami)) to use for the instance
+       - I(ami) ID to use for the instance
     required: true
     default: null
     aliases: []
@@ -94,7 +95,7 @@ options:
     aliases: []
   wait:
     description:
-      - wait for the instance to be in state 'running' before returning
+      - wait for the instance to be 'running' before returning.  Does not wait for SSH, see 'wait_for' example for details.
     required: false
     default: "no"
     choices: [ "yes", "no" ]
@@ -226,54 +227,55 @@ extends_documentation_fragment: aws
 '''
 
 EXAMPLES = '''
-# Note: None of these examples set aws_access_key, aws_secret_key, or region.
-# It is assumed that their matching environment variables are set.
+# Note: These examples do not set authentication details, see the AWS Guide for details.
 
 # Basic provisioning example
-- local_action:
-    module: ec2
+- ec2:
     key_name: mykey
-    instance_type: c1.medium
-    image: emi-40603AD1
+    instance_type: t2.micro
+    image: ami-123456
     wait: yes
     group: webserver
     count: 3
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 # Advanced example with tagging and CloudWatch
-- local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     group: databases
-    instance_type: m1.large
-    image: ami-6e649707
+    instance_type: t2.micro
+    image: ami-123456
     wait: yes
     wait_timeout: 500
     count: 5
     instance_tags: 
        db: postgres
     monitoring: yes
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 # Single instance with additional IOPS volume from snapshot and volume delete on termination
-local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     group: webserver
-    instance_type: m1.large
-    image: ami-6e649707
+    instance_type: c3.medium
+    image: ami-123456
     wait: yes
     wait_timeout: 500
     volumes:
-    - device_name: /dev/sdb
-      snapshot: snap-abcdef12
-      device_type: io1
-      iops: 1000
-      volume_size: 100
-      delete_on_termination: true
+      - device_name: /dev/sdb
+        snapshot: snap-abcdef12
+        device_type: io1
+        iops: 1000
+        volume_size: 100
+        delete_on_termination: true
     monitoring: yes
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 # Multiple groups example
-local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     group: ['databases', 'internal-services', 'sshable', 'and-so-forth']
     instance_type: m1.large
@@ -284,10 +286,11 @@ local_action:
     instance_tags: 
         db: postgres
     monitoring: yes
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 # Multiple instances with additional volume from snapshot
-local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     group: webserver
     instance_type: m1.large
@@ -300,21 +303,23 @@ local_action:
       snapshot: snap-abcdef12
       volume_size: 10
     monitoring: yes
-
-# VPC example
-- local_action:
-    module: ec2
-    key_name: mykey
-    group_id: sg-1dc53f72
-    instance_type: m1.small
-    image: ami-6e649707
-    wait: yes
     vpc_subnet_id: subnet-29e63245
     assign_public_ip: yes
 
-# Spot instance example
+# Dedicated tenancy example
 - local_action:
     module: ec2
+    assign_public_ip: yes
+    group_id: sg-1dc53f72
+    key_name: mykey
+    image: ami-6e649707
+    instance_type: m1.small
+    tenancy: dedicated
+    vpc_subnet_id: subnet-29e63245
+    wait: yes
+
+# Spot instance example
+- ec2:
     spot_price: 0.24
     spot_wait_timeout: 600
     keypair: mykey
@@ -328,7 +333,6 @@ local_action:
 # Launch instances, runs some tasks
 # and then terminate them
 
-
 - name: Create a sandbox instance
   hosts: localhost
   gather_facts: False
@@ -340,13 +344,21 @@ local_action:
     region: us-east-1
   tasks:
     - name: Launch instance
-      local_action: ec2 key_name={{ keypair }} group={{ security_group }} instance_type={{ instance_type }} image={{ image }} wait=true region={{ region }}
+      ec2: 
+         key_name: "{{ keypair }}"
+         group: "{{ security_group }}"
+         instance_type: "{{ instance_type }}"
+         image: "{{ image }}"
+         wait: true
+         region: "{{ region }}"
+         vpc_subnet_id: subnet-29e63245
+         assign_public_ip: yes
       register: ec2
     - name: Add new instance to host group
-      local_action: add_host hostname={{ item.public_ip }} groupname=launched
+      add_host: hostname={{ item.public_ip }} groupname=launched
       with_items: ec2.instances
     - name: Wait for SSH to come up
-      local_action: wait_for host={{ item.public_dns_name }} port=22 delay=60 timeout=320 state=started
+      wait_for: host={{ item.public_dns_name }} port=22 delay=60 timeout=320 state=started
       with_items: ec2.instances
 
 - name: Configure instance(s)
@@ -362,8 +374,7 @@ local_action:
   connection: local
   tasks:
     - name: Terminate instances that were previously launched
-      local_action:
-        module: ec2
+      ec2:
         state: 'absent'
         instance_ids: '{{ ec2.instance_ids }}'
 
@@ -382,12 +393,13 @@ local_action:
     region: us-east-1
   tasks:
     - name: Start the sandbox instances
-      local_action:
-        module: ec2
+      ec2:
         instance_ids: '{{ instance_ids }}'
         region: '{{ region }}'
         state: running
         wait: True
+        vpc_subnet_id: subnet-29e63245
+        assign_public_ip: yes
   role:
     - do_neat_stuff
     - do_more_neat_stuff
@@ -404,38 +416,40 @@ local_action:
     region: us-east-1
   tasks:
     - name: Stop the sandbox instances
-      local_action:
-      module: ec2
-      instance_ids: '{{ instance_ids }}'
-      region: '{{ region }}'
-      state: stopped
-      wait: True
+      ec2:
+        instance_ids: '{{ instance_ids }}'
+        region: '{{ region }}'
+        state: stopped
+        wait: True
+        vpc_subnet_id: subnet-29e63245
+        assign_public_ip: yes
 
 #
 # Enforce that 5 instances with a tag "foo" are running
+# (Highly recommended!)
 #
 
-- local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     instance_type: c1.medium
-    image: emi-40603AD1
+    image: ami-40603AD1
     wait: yes
     group: webserver
     instance_tags:
         foo: bar
     exact_count: 5
     count_tag: foo
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 #
 # Enforce that 5 running instances named "database" with a "dbtype" of "postgres"
 #
 
-- local_action:
-    module: ec2
+- ec2:
     key_name: mykey
     instance_type: c1.medium
-    image: emi-40603AD1
+    image: ami-40603AD1
     wait: yes
     group: webserver
     instance_tags: 
@@ -445,6 +459,8 @@ local_action:
     count_tag: 
         Name: database
         dbtype: postgres
+    vpc_subnet_id: subnet-29e63245
+    assign_public_ip: yes
 
 #
 # count_tag complex argument examples
@@ -479,6 +495,7 @@ try:
     import boto.ec2
     from boto.ec2.blockdevicemapping import BlockDeviceType, BlockDeviceMapping
     from boto.exception import EC2ResponseError
+    from boto.vpc import VPCConnection
 except ImportError:
     print "failed=True msg='boto required for this module'"
     sys.exit(1)
@@ -574,7 +591,10 @@ def get_instance_info(inst):
                      'root_device_type': inst.root_device_type,
                      'root_device_name': inst.root_device_name,
                      'state': inst.state,
-                     'hypervisor': inst.hypervisor}
+                     'hypervisor': inst.hypervisor,
+                     'tags': inst.tags,
+                     'groups': dict((group.id, group.name) for group in inst.groups),
+                     }
     try:
         instance_info['virtualization_type'] = getattr(inst,'virtualization_type')
     except AttributeError:
@@ -585,6 +605,11 @@ def get_instance_info(inst):
     except AttributeError:
         instance_info['ebs_optimized'] = False
 
+    try:
+        instance_info['tenancy'] = getattr(inst, 'placement_tenancy')
+    except AttributeError:
+        instance_info['tenancy'] = 'default'
+
     return instance_info
 
 def boto_supports_associate_public_ip_address(ec2):
@@ -654,7 +679,7 @@ def boto_supports_param_in_spot_request(ec2, param):
     method = getattr(ec2, 'request_spot_instances')
     return param in method.func_code.co_varnames
 
-def enforce_count(module, ec2):
+def enforce_count(module, ec2, vpc):
 
     exact_count = module.params.get('exact_count')
     count_tag = module.params.get('count_tag')
@@ -679,7 +704,7 @@ def enforce_count(module, ec2):
         to_create = exact_count - len(instances)
         if not checkmode:
             (instance_dict_array, changed_instance_ids, changed) \
-                = create_instances(module, ec2, override_count=to_create)
+                = create_instances(module, ec2, vpc, override_count=to_create)
 
             for inst in instance_dict_array:
                 instances.append(inst)
@@ -710,7 +735,7 @@ def enforce_count(module, ec2):
     return (all_instances, instance_dict_array, changed_instance_ids, changed)
     
         
-def create_instances(module, ec2, override_count=None):
+def create_instances(module, ec2, vpc, override_count=None):
     """
     Creates new instances
 
@@ -728,6 +753,7 @@ def create_instances(module, ec2, override_count=None):
     group_id = module.params.get('group_id')
     zone = module.params.get('zone')
     instance_type = module.params.get('instance_type')
+    tenancy = module.params.get('tenancy')
     spot_price = module.params.get('spot_price')
     image = module.params.get('image')
     if override_count:
@@ -758,25 +784,29 @@ def create_instances(module, ec2, override_count=None):
         module.fail_json(msg = str("Use only one type of parameter (group_name) or (group_id)"))
         sys.exit(1)
 
+    vpc_id = None
+    if vpc_subnet_id:
+        vpc_id = vpc.get_all_subnets(subnet_ids=[vpc_subnet_id])[0].vpc_id
+    else:
+        vpc_id = None
+
     try:
         # Here we try to lookup the group id from the security group name - if group is set.
         if group_name:
-            grp_details = ec2.get_all_security_groups()
-            if type(group_name) == list:
-                group_id = [ str(grp.id) for grp in grp_details if str(grp.name) in group_name ]
-            elif type(group_name) == str:
-                for grp in grp_details:
-                    if str(group_name) in str(grp):
-                        group_id = [str(grp.id)]
+            if vpc_id:
+                grp_details = ec2.get_all_security_groups(filters={'vpc_id': vpc_id})
+            else:
+                grp_details = ec2.get_all_security_groups()
+            if isinstance(group_name, basestring):
                 group_name = [group_name]
+            group_id = [ str(grp.id) for grp in grp_details if str(grp.name) in group_name ]
         # Now we try to lookup the group id testing if group exists.
         elif group_id:
             #wrap the group_id in a list if it's not one already
-            if type(group_id) == str:
+            if isinstance(group_id, basestring):
                 group_id = [group_id]
             grp_details = ec2.get_all_security_groups(group_ids=group_id)
-            grp_item = grp_details[0]
-            group_name = [grp_item.name]
+            group_name = [grp_item.name for grp_item in grp_details]
     except boto.exception.NoAuthHandlerFound, e:
             module.fail_json(msg = str(e))
 
@@ -811,6 +841,10 @@ def create_instances(module, ec2, override_count=None):
 
             if ebs_optimized:
               params['ebs_optimized'] = ebs_optimized
+              
+            # 'tenancy' always has a default value, but it is not a valid parameter for spot instance resquest
+            if not spot_price:
+              params['tenancy'] = tenancy
 
             if boto_supports_profile_name_arg(ec2):
                 params['instance_profile_name'] = instance_profile_name
@@ -890,6 +924,18 @@ def create_instances(module, ec2, override_count=None):
                             continue
                         else:
                             module.fail_json(msg = str(e))
+
+                # The instances returned through ec2.run_instances above can be in
+                # terminated state due to idempotency. See commit 7f11c3d for a complete
+                # explanation.
+                terminated_instances = [
+                    str(instance.id) for instance in res.instances if instance.state == 'terminated'
+                ]
+                if terminated_instances:
+                    module.fail_json(msg = "Instances with id(s) %s " % terminated_instances +
+                                           "were created previously but have since been terminated - " +
+                                           "use a (possibly different) 'instanceid' parameter")
+
             else:
                 if private_ip:
                     module.fail_json(
@@ -927,15 +973,6 @@ def create_instances(module, ec2, override_count=None):
         except boto.exception.BotoServerError, e:
             module.fail_json(msg = "Instance creation failed => %s: %s" % (e.error_code, e.error_message))
 
-        # The instances returned through run_instances can be in
-        # terminated state due to idempotency.
-        terminated_instances = [ str(instance.id) for instance in res.instances
-                                 if instance.state == 'terminated' ]
-        if terminated_instances:
-            module.fail_json(msg = "Instances with id(s) %s " % terminated_instances +
-                                   "were created previously but have since been terminated - " +
-                                   "use a (possibly different) 'instanceid' parameter")
-
         # wait here until the instances are up
         num_running = 0
         wait_timeout = time.time() + wait_timeout
@@ -1153,6 +1190,7 @@ def main():
             count_tag = dict(),
             volumes = dict(type='list'),
             ebs_optimized = dict(type='bool', default=False),
+            tenancy = dict(default='default'),
         )
     )
 
@@ -1167,6 +1205,20 @@ def main():
 
     ec2 = ec2_connect(module)
 
+    ec2_url, aws_access_key, aws_secret_key, region = get_ec2_creds(module)
+
+    if region:
+        try:
+            vpc = boto.vpc.connect_to_region(
+                region,
+                aws_access_key_id=aws_access_key,
+                aws_secret_access_key=aws_secret_key
+            )
+        except boto.exception.NoAuthHandlerFound, e:
+            module.fail_json(msg = str(e))
+    else:
+        module.fail_json(msg="region must be specified")
+
     tagged_instances = [] 
 
     state = module.params.get('state')
@@ -1191,9 +1243,9 @@ def main():
             module.fail_json(msg='image parameter is required for new instance')
 
         if module.params.get('exact_count') is None:
-            (instance_dict_array, new_instance_ids, changed) = create_instances(module, ec2)
+            (instance_dict_array, new_instance_ids, changed) = create_instances(module, ec2, vpc)
         else:
-            (tagged_instances, instance_dict_array, new_instance_ids, changed) = enforce_count(module, ec2)
+            (tagged_instances, instance_dict_array, new_instance_ids, changed) = enforce_count(module, ec2, vpc)
 
     module.exit_json(changed=changed, instance_ids=new_instance_ids, instances=instance_dict_array, tagged_instances=tagged_instances)
 
