@@ -1495,7 +1495,8 @@ def main():
                          msg=manager.get_summary_message(),
                          summary=manager.counters,
                          containers=containers.changed,
-                         reload_reasons=manager.get_reload_reason_message())
+                         reload_reasons=manager.get_reload_reason_message(),
+                         ansible_facts=_ansible_facts(containers.changed))
 
     except DockerAPIError as e:
         module.fail_json(changed=manager.has_changed(), msg="Docker API Error: %s" % e.explanation)
