@@ -472,7 +472,7 @@ class LinuxService(Service):
             self.module.fail_json(msg="no service or tool found for: %s" % self.name)
 
         # If no service control tool selected yet, try to see if 'service' is available
-        if not self.svc_cmd and location.get('service', False):
+        if self.svc_cmd is None and location.get('service', False):
             self.svc_cmd = location['service']
 
         # couldn't find anything yet
@@ -703,14 +703,10 @@ class LinuxService(Service):
 
             # Check if we're already in the correct state
             service_enabled = self.get_systemd_service_enabled()
-            if self.enable and service_enabled:
-                self.changed = False
-            elif not self.enable and not service_enabled:
-                self.changed = False
-            elif not self.enable:
-                self.changed = False
 
-            if not self.changed:
+            # self.changed should already be true
+            if self.enable == service_enabled:
+                self.changed = False
                 return
 
         #
