@@ -54,7 +54,7 @@ version_added: "0.7"
 options:
   host:
     description:
-      - hostname or IP address to wait for
+      - A resolvable hostname or IP address to wait for
     required: false
     default: "127.0.0.1"
     aliases: []
@@ -63,6 +63,11 @@ options:
       - maximum number of seconds to wait for
     required: false
     default: 300
+  connect_timeout:
+    description:
+      - maximum number of seconds to wait for a connection to happen before closing and retrying
+    required: false
+    default: 5
   delay:
     description:
       - number of seconds to wait before starting to poll
@@ -123,8 +128,9 @@ EXAMPLES = '''
 # wait until the process is finished and pid was destroyed
 - wait_for: path=/proc/3466/status state=absent
 
-# Wait 300 seconds for port 22 to become open and contain "OpenSSH", don't start checking for 10 seconds
-- local_action: wait_for port=22 host="{{ inventory_hostname }}" search_regex=OpenSSH delay=10
+# wait 300 seconds for port 22 to become open and contain "OpenSSH", don't assume the inventory_hostname is resolvable
+# and don't start checking for 10 seconds
+- local_action: wait_for port=22 host="{{ ansible_ssh_host | default(inventory_hostname) }}" search_regex=OpenSSH delay=10
 
 '''
 
